@@ -1,12 +1,15 @@
 package Net::Hacky::Detect::IP;
 
+# Base modules
 use 5.006;
 use strict;
 use warnings;
+
+use File::Temp qw( tempfile );
+use IO::Socket::INET;
+use IO::Socket::INET6;
 use Try::Tiny;
 use Capture::Tiny ':all';
-use IO::Socket::INET;
-use File::Temp qw( tempfile );
 
 =head1 NAME
 
@@ -16,20 +19,13 @@ Net::Hacky::Detect::IP - Hackily try different methods of attaining local system
 
 This module does not yet support returning IPv6 addresses, though will do soon!
 
-=head1 README
-
-I am working on a lot cleaner version of this module, that uses the detected OS and acts from there, however I do not havea
-access to a vast amount of machines, if you would like to help me, please read 'Bugs' and use the corrosponding tracker, to
-submit: Your OS, What binaries are on the system for showing ip information, the output of the command, the output of 'uname -a'
-and any other relevant data.
-
 =head1 VERSION
 
-Version 0.014
+Version 0.015
 
 =cut
 
-our $VERSION = '0.014';
+our $VERSION = '0.015';
 
 my $tools = {
     unix => {
@@ -62,9 +58,15 @@ EOF
 =head1 SYNOPSIS
 
     use Net::Hacky::Detect::IP;
-    my @ips = @{ Net::Hacky::Detect::IP->scan() };
-    foreach my $ip (@ips) {
-        print "Detected ip: $ip\n";
+    my $ip = Net::Hacky::Detect::IP->new();
+
+    my @ips = @{ $ip->scan() };
+
+    foreach my $host (@ips) {
+        print "Detected ip: $host\n";
+        if ($ip->checkIP($host)) {
+            print "IP is deffinetly usable! (Was tested twice as scan() auto checks)\n";
+        }
     }
 
 =head1 DESCRIPTION 
@@ -75,13 +77,34 @@ however you go about it and this method appears to be the more reliable.
 
 =head1 METHODS
 
+=head2 new
+
+Create a new object for interacting with Net-Hacky-Detect-IP
+
+=cut 
+
+sub new {
+    my ($class,$PND,$AutoStart,$debug) = @_;
+
+    # Some private stuff for ourself
+    my $self = {
+    };
+
+    # Go with god my son
+    bless $self, $class;
+    return $self;
+}
+
+
 =head2 scan
 
-Attempt to find local system ips, returns a list of [] if nothing found.
+Attempt to find local system ips, returns a list of local ips will return a [] black list if nothing found.
 
 =cut
 
 sub scan {
+    my $self = shift;
+
     my $return = [];
     my $os = 'unix';
     
@@ -90,7 +113,6 @@ sub scan {
     # Some short cuts and initial scalars for storing things in
     my $dumps = "";
     my $short = $tools->{$os};
-    my $winScript;
 
     # Go searching for something we can use
     foreach my $tool ( @{ $short->{tools} } ) {
@@ -104,11 +126,15 @@ sub scan {
 
             # If this is a call for cscript, we need to act differently..
             if ($cmd eq 'cscript') {
-                $winScript = File::Temp::tempdir() . '\\' . 'findip.vbs';
+                # Generate a path to a writable space
+                my $winScript = File::Temp::tempdir() . '\\' . 'findip.vbs';
+                
+                # Write our vbs to that location
                 open(my $fh,'>',$winScript);
                 print $fh $cscript;
                 close($fh);
 
+                # Push the filepath into the arguments
                 push @args,$winScript;
             }
             
@@ -146,7 +172,19 @@ sub scan {
             # Check we have not already dealt with this ip and its valid
             next if ($unique->{$IP});
             $unique->{$IP} = 1;
-            if (!_checkIPv4(4,$IP)) { next }
+            if (!$self->_checkIP4($IP)) { next }
+
+            # Push the valid ip into the return space
+            push @$return,$IP;
+        }
+        if ($testIP =~ m#:# && $testIP =~ m#[0-9]# && $testIP =~ m#^([a-f0-9:]+)$#) {
+            # Copy $1 to $IP because it looks prettier
+            my $IP = $1;
+            
+            # Check we have not already dealt with this ip and its valid
+            next if ($unique->{$IP});
+            $unique->{$IP} = 1;
+            if (!$self->_checkIP6($IP)) { next }
 
             # Push the valid ip into the return space
             push @$return,$IP;
@@ -155,57 +193,102 @@ sub scan {
     return $return;
 }
 
-=head1 INTERNAL METHODFS
+=head2 checkIP 
 
-=head2 _checkIPv4
+Check an IP is valid and usable, takes 1 mandatory argument;
 
-Check an IPv4 is valid and usable, takes 2 mandatory arguments;
+Argument 1 Should be an ip address (not hostname)
 
-a '4' or '6' to designate an IPv4 or IPv6 address and the host its self as an ip not hostname.
+Returns 1 on success 'valid' and 0 on failure 'invalid'
 
 =cut
 
+sub checkIP {
+    my ($self,$host) = @_;
 
-sub _checkIPv4 {
-    my ($version,$host) = @_;
+    if ($host =~ m#\.#) { 
+        return $self->_checkIP4($host);
+    } 
+    elsif ($host =~ m#:#) {
+        return $self->_checkIP6($host);
+    }
+    else { 
+        warn "$host does not look like an IPv6 nor IPv4, ignored.";
+    }
+}
+
+
+sub _checkIP4 {
+    my ($self,$host) = @_;
     
-    if (!$version || !$host) {
+    # A few safety checks
+    if (!$host) {
         warn "Incorrect number of arguments, returning fail";
         return 0;
     }
 
     # By default fail the bind
-    my $bindfail = 0;
+    my $bindsuccess = 0;
     
-    if ($version == 4) {
-        # Split the ip into relevent blocks
-        my @ip = split(/\./,$host);
+    # Split the ip into relevent blocks
+    my @ip = split(/\./,$host);
 
-        # Do a more precise check (This should rule out all netmasks and broadcasts)
-        return 0 if ($ip[0] <= 0 || $ip[0] >= 255);
-        return 0 if ($ip[1] < 0 || $ip[1] > 255);
-        return 0 if ($ip[2] < 0 || $ip[2] > 255);
-        return 0 if ($ip[3] <= 0 || $ip[3] >= 255);
-        
-        # Now lets try listen on the ip.. see if we can bind to it, try three times on
-        # random ports incase we bump into a busy one.
-        for (1..3) {
-            my $fail=1;
-            my $port = 1024+int(rand(65410));
-            my $sock = IO::Socket::INET->new(
-                LocalAddr   =>  $host,
-                LocalPort   =>  $port,
-                Proto       =>  'tcp',
-                ReuseAddr   =>  1
-            ) or $fail = 0;
-            if ($fail) {
-                $bindfail=1;
-                last;
-            }
+    # Do a more precise check (This should rule out all netmasks and broadcasts)
+    return 0 if ($ip[0] <= 0 || $ip[0] >= 255);
+    return 0 if ($ip[1] < 0 || $ip[1] > 255);
+    return 0 if ($ip[2] < 0 || $ip[2] > 255);
+    return 0 if ($ip[3] <= 0 || $ip[3] >= 255);
+    
+    # Now lets try listen on the ip.. see if we can bind to it, try three times on
+    # random ports incase we bump into a busy one.
+    for (1..3) {
+        my $fail=1;
+        my $port = 1025+int(rand(64510));
+        my $sock = IO::Socket::INET->new(
+            LocalAddr   =>  $host,
+            LocalPort   =>  $port,
+            Proto       =>  'tcp',
+            ReuseAddr   =>  1
+        ) or $fail = 0;
+        if ($fail) {
+            $bindsuccess=1;
+            last;
+        }
+    }
+
+    return $bindsuccess;
+}
+
+sub _checkIP6 {
+    my ($self,$host) = @_;
+    
+    # A few safety checks
+    if (!$host) {
+        warn "Incorrect number of arguments, returning fail";
+        return 0;
+    }
+
+    # By default fail the bind
+    my $bindsuccess = 0;
+
+   # Now lets try listen on the ip.. see if we can bind to it, try three times on
+    # random ports incase we bump into a busy one.
+    for (1..3) {
+        my $fail=1;
+        my $port = 1025+int(rand(64510));
+        my $sock = IO::Socket::INET6->new(
+            LocalAddr   =>  $host,
+            LocalPort   =>  $port,
+            Proto       =>  'tcp',
+            ReuseAddr   =>  1
+        ) or $fail = 0;
+        if ($fail) {
+            $bindsuccess=1;
+            last;
         }
     }
     
-    return $bindfail;
+    return $bindsuccess;
 }
 
 
@@ -215,7 +298,7 @@ Paul G Webster, C<< <daemon at cpan.org> >>
 
 =head1 BUGS
 
-Please report any bugs or feature requests through the authors code repository at c<https://gitlab.com/paul-g-webster/PL-Net-Hacky-Detect-IP>
+Please report any bugs or feature requests through the authors code repository at C<https://gitlab.com/paul-g-webster/PL-Net-Hacky-Detect-IP>
 
 =head1 SUPPORT
 
@@ -228,6 +311,10 @@ You can also look for information at:
 
 =over 4
 
+=item * GitLab: The authors gitlab page for this project
+
+L<https://gitlab.com/paul-g-webster/PL-Net-Hacky-Detect-IP>
+
 =item * AnnoCPAN: Annotated CPAN documentation
 
 L<http://annocpan.org/dist/Net-Hacky-Detect-IP>
@@ -245,6 +332,7 @@ L<http://search.cpan.org/dist/Net-Hacky-Detect-IP/>
 
 =head1 ACKNOWLEDGEMENTS
 
+Thank you for all the continued help from irc.freenode.net #perl and irc.perl.org #perl
 
 =head1 LICENSE AND COPYRIGHT
 
